;
; Simple sprite demo for Game Boy
;
; Copyright 2018 Damian Yerrick
; 
; This software is provided 'as-is', without any express or implied
; warranty.  In no event will the authors be held liable for any damages
; arising from the use of this software.
; 
; Permission is granted to anyone to use this software for any purpose,
; including commercial applications, and to alter it and redistribute it
; freely, subject to the following restrictions:
; 
; 1. The origin of this software must not be misrepresented; you must not
;    claim that you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation would be
;    appreciated but is not required.
; 2. Altered source versions must be plainly marked as such, and must not be
;    misrepresented as being the original software.
; 3. This notice may not be removed or altered from any source distribution.
;
include "src/hardware.inc"
include "src/global.inc"

section "rom_main", ROM0

; Double-colon these so that the header can see them
stat_handler::
timer_handler::
serial_handler::
joypad_handler::
  reti

main::
  ; Avoid uninitialized read warning for nmis
  xor a
  ld [nmis],a

  ; Set up vblank handler
  ld a,IEF_VBLANK
  ldh [rIE],a  ; enable IRQs
  xor a
  ldh [rIF],a  ; Acknowledge any pending IRQs
  ei

  call draw_bg
  call load_player_cels

  ; Blank GB palettes
  xor a
  ldh [rBGP],a
  ldh [rOBP0],a
  ldh [rOBP1],a

  ; Blank GBC palettes
  ld a,$80
  ldh [rBCPS],a  ; Background palette address
  ldh [rOCPS],a  ; Sprite palette address
  ld a,$FF  ; white
  ld b,64
  .blankpalloop:
    ldh [rBCPD],a
    ldh [rOCPD],a
    dec b
    jr nz,.blankpalloop
  
  ; Can turn on rendering at any time.  Only turning off
  ; rendering is time restricted.  GB LCD freesync is backwards
  ; from traditional NES/Super NES video in this way.
  ; Unlike NES and Super NES, Game Boy asserts no vblank IRQ in
  ; forced blanking, so turn on the LCD but blank the palette.
  ld a,LCDCF_ON|OBJ_ON|BG_NT0|BG_CHR21
  ldh [rLCDC],a

.loop:
  call read_pad
  call move_player

  ; First make sure it doesn't crash
  xor a
  ld [oam_used],a
  call draw_player_sprite
  call lcd_clear_oam

  ; Push sprite display list to OAM
  call wait_vblank_irq
  call run_dma

  ; Set GB palettes
  ld a,%01101100
  ldh [rBGP],a
  ld a,%00011110
  ldh [rOBP0],a

  ; Set GBC palettes
  ld a,$80
  ldh [rBCPS],a
  ldh [rOCPS],a
  ld hl,bg_palette
  ld b,16
  .bgpalloop:
    ld a,[hl+]
    ldh [rBCPD],a
    dec b
    jr nz,.bgpalloop
  ld hl,obj_palette
  ld b,8
  .objpalloop:
    ld a,[hl+]
    ldh [rOCPD],a
    dec b
    jr nz,.objpalloop

  ; Show vblank counter
  ld hl,_SCRN0
  ld a,[nmis]
  call puthex

  jr .loop

bg_palette:
  drgb $8fa1ff  ; 0: Sky and floor
  drgb $342800
  drgb $366d00
  drgb $55c753
  drgb $8fa1ff  ; 1: Push blocks
  drgb $6D5C00
  drgb $BDAC2C
  drgb $E4DCA8

obj_palette:
  drgb $000000
  drgb $342800
  drgb $BD3C30
  drgb $FFCFCA
